@@ -1,7 +1,6 @@
 import os
 import pickle
 import uuid
-from contextlib import contextmanager
 
 import six
 
@@ -14,21 +13,42 @@ from dagster import (
     SolidDefinition,
     TypeCheck,
     check,
+    seven,
 )
 from dagster.cli import load_handle
 from dagster.core.definitions.dependency import SolidHandle
-from dagster.core.execution.api import scoped_pipeline_context
-from dagster.core.execution.context_creation_pipeline import ResourcesStack
+from dagster.core.definitions.resource import ScopedResourcesBuilder
+from dagster.core.execution.api import create_execution_plan, scoped_pipeline_context
+from dagster.core.execution.resources_init import (
+    get_required_resource_keys_to_init,
+    resource_initialization_event_generator,
+)
 from dagster.core.instance import DagsterInstance
-from dagster.core.serdes import unpack_value
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
+from dagster.core.utils import make_new_run_id
 from dagster.loggers import colored_console_logger
+from dagster.serdes import unpack_value
+from dagster.utils import EventGenerationManager
 
-from .context import DagstermillExecutionContext
+from .context import DagstermillExecutionContext, DagstermillRuntimeExecutionContext
 from .errors import DagstermillError
 from .serialize import PICKLE_PROTOCOL, read_value, write_value
 
 
+class DagstermillResourceEventGenerationManager(EventGenerationManager):
+    ''' Utility class to explicitly manage setup/teardown of resource events. Overrides the default
+    `generate_teardown_events` method so that teardown is deferred until explicitly called by the
+    dagstermill Manager
+    '''
+
+    def generate_teardown_events(self):
+        return iter(())
+
+    def teardown(self):
+        for _ in super(DagstermillResourceEventGenerationManager, self).generate_teardown_events():
+            pass
+
+
 class Manager(object):
     def __init__(self):
         self.handle = None
@@ -37,21 +57,23 @@ class Manager(object):
         self.in_pipeline = False
         self.marshal_dir = None
         self.context = None
-        self.resources_stack = None
+        self.resource_manager = None
 
-    @contextmanager
-    def _setup_resources(self, pipeline_def, environment_config, pipeline_run, log_manager):
-        '''This context manager is a drop-in replacement for
-        dagster.core.execution.context_creation_pipeline.create_resources. It uses the Manager's
-        instance of ResourceStack to create resources, but does not tear them down when the
-        context manager returns -- teardown must be managed manually using Manager.teardown().
+    def _setup_resources(
+        self, execution_plan, environment_config, pipeline_run, log_manager, resource_keys_to_init
+    ):
         '''
-
-        # pylint: disable=protected-access
-        self.resources_stack = ResourcesStack(
-            pipeline_def, environment_config, pipeline_run, log_manager
+        Drop-in replacement for
+        `dagster.core.execution.resources_init.resource_initialization_manager`.  It uses a
+        `DagstermillResourceEventGenerationManager` and explicitly calls `teardown` on it
+        '''
+        generator = resource_initialization_event_generator(
+            execution_plan, environment_config, pipeline_run, log_manager, resource_keys_to_init
         )
-        yield self.resources_stack.create()
+        self.resource_manager = DagstermillResourceEventGenerationManager(
+            generator, ScopedResourcesBuilder
+        )
+        return self.resource_manager
 
     def reconstitute_pipeline_context(
         self,
@@ -117,7 +139,7 @@ class Manager(object):
         ).build_sub_pipeline(solid_subset)
 
         solid_handle = SolidHandle.from_dict(solid_handle_kwargs)
-        solid_def = pipeline_def.get_solid(solid_handle)
+        solid_def = pipeline_def.get_solid(solid_handle).definition
 
         pipeline_run = unpack_value(pipeline_run_dict)
 
@@ -126,14 +148,28 @@ class Manager(object):
         self.solid_def = solid_def
         self.pipeline_def = pipeline_def
 
+        execution_plan = create_execution_plan(
+            self.pipeline_def,
+            environment_dict,
+            mode=pipeline_run.mode,
+            step_keys_to_execute=pipeline_run.step_keys_to_execute,
+        )
+
         with scoped_pipeline_context(
             self.pipeline_def,
             environment_dict,
             pipeline_run,
-            instance=instance,
+            instance,
+            execution_plan,
             scoped_resources_builder_cm=self._setup_resources,
         ) as pipeline_context:
-            self.context = DagstermillExecutionContext(pipeline_context)
+            self.context = DagstermillRuntimeExecutionContext(
+                pipeline_context=pipeline_context,
+                solid_config=None,
+                resource_keys_to_init=get_required_resource_keys_to_init(
+                    execution_plan, pipeline_context.system_storage_def
+                ),
+            )
 
         return self.context
 
@@ -151,28 +187,36 @@ class Manager(object):
                 the context.
 
         Returns:
-            :class:`dagstermill.DagstermillExecutionContext`
+            :py:class:`~dagstermill.DagstermillExecutionContext`
         '''
         check.opt_inst_param(mode_def, 'mode_def', ModeDefinition)
         environment_dict = check.opt_dict_param(environment_dict, 'environment_dict', key_type=str)
 
+        # If we are running non-interactively, and there is already a context reconstituted, return
+        # that context rather than overwriting it.
+        if self.context is not None and isinstance(
+            self.context, DagstermillRuntimeExecutionContext
+        ):
+            return self.context
+
+        if not mode_def:
+            mode_def = ModeDefinition(logger_defs={'dagstermill': colored_console_logger})
+            environment_dict['loggers'] = {'dagstermill': {}}
+
         solid_def = SolidDefinition(
             name='this_solid',
             input_defs=[],
             compute_fn=lambda *args, **kwargs: None,
             output_defs=[],
             description='Ephemeral solid constructed by dagstermill.get_context()',
+            required_resource_keys=mode_def.resource_key_set,
         )
 
-        if not mode_def:
-            mode_def = ModeDefinition(logger_defs={'dagstermill': colored_console_logger})
-            environment_dict['loggers'] = {'dagstermill': {}}
-
         pipeline_def = PipelineDefinition(
             [solid_def], mode_defs=[mode_def], name='ephemeral_dagstermill_pipeline'
         )
 
-        run_id = str(uuid.uuid4())
+        run_id = make_new_run_id()
 
         # construct stubbed PipelineRun for notebook exploration...
         # The actual pipeline run during pipeline execution will be serialized and reconstituted
@@ -182,7 +226,6 @@ class Manager(object):
             run_id=run_id,
             environment_dict=environment_dict,
             mode=mode_def.name,
-            reexecution_config=None,
             selector=None,
             step_keys_to_execute=None,
             status=PipelineRunStatus.NOT_STARTED,
@@ -193,14 +236,25 @@ class Manager(object):
         self.solid_def = solid_def
         self.pipeline_def = pipeline_def
 
+        execution_plan = create_execution_plan(
+            self.pipeline_def, environment_dict, mode=mode_def.name
+        )
         with scoped_pipeline_context(
             self.pipeline_def,
             environment_dict,
             pipeline_run,
-            instance=DagsterInstance.ephemeral(),
+            DagsterInstance.ephemeral(),
+            execution_plan,
             scoped_resources_builder_cm=self._setup_resources,
         ) as pipeline_context:
-            self.context = DagstermillExecutionContext(pipeline_context, solid_config)
+
+            self.context = DagstermillExecutionContext(
+                pipeline_context=pipeline_context,
+                solid_config=solid_config,
+                resource_keys_to_init=get_required_resource_keys_to_init(
+                    execution_plan, pipeline_context.system_storage_def
+                ),
+            )
 
         return self.context
 
@@ -226,10 +280,10 @@ class Manager(object):
                 )
             )
 
-        runtime_type = self.solid_def.output_def_named(output_name).runtime_type
+        dagster_type = self.solid_def.output_def_named(output_name).dagster_type
 
         out_file = os.path.join(self.marshal_dir, 'output-{}'.format(output_name))
-        scrapbook.glue(output_name, write_value(runtime_type, value, out_file))
+        scrapbook.glue(output_name, write_value(dagster_type, value, out_file))
 
     def yield_event(self, dagster_event):
         '''Yield a dagster event directly from notebook code.
@@ -258,12 +312,12 @@ class Manager(object):
         scrapbook.glue(event_id, out_file_path)
 
     def teardown_resources(self):
-        if self.resources_stack is not None:
-            self.resources_stack.teardown()
+        if self.resource_manager is not None:
+            self.resource_manager.teardown()
 
     def load_parameter(self, input_name, input_value):
         input_def = self.solid_def.input_def_named(input_name)
-        return read_value(input_def.runtime_type, input_value)
+        return read_value(input_def.dagster_type, seven.json.loads(input_value))
 
 
 MANAGER_FOR_NOTEBOOK_INSTANCE = Manager()
